

CREATE OR REPLACE FUNCTION fHutangDate (
	_Tipe		INT,
	_Date		DATE,
	_CrcID		VARCHAR,
	_NotEqualZero	BOOLEAN
) RETURNS SETOF tHutangPiutang AS
$BODY$
DECLARE
	_retval		tHutangPiutang%ROWTYPE;

	_BPID		VARCHAR(15);
	_BPName		VARCHAR(50);
	_Balance	NUMERIC(19,4);
	_CrcName	VARCHAR(50);
	_CrcSymbol	VARCHAR(5);
	_CurrCrcID	VARCHAR;

	__Date		DATE;

	_PYear		INT2;
	_Month		INT2;
	_Day		INT2;
	_StartDate	DATE;

	__Balance	NUMERIC(19,4);
	_FCNBalance	NUMERIC(19,4);

	cBal		REFCURSOR;
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month;

	OPEN cBal FOR
		SELECT DISTINCT bp.bpid, bpname, fBPBalanceDate(0, bp.bpid, crc.crcid, _Month, _PYear) AS apbalance, crcname, crcsymbol, crc.crcid  
		FROM bp 
		LEFT JOIN bpacc ON bp.bpid=bpacc.bpid 
		JOIN bpbptype bt ON bt.bpid=bp.bpid
		JOIN crc ON crc.crcid=bpacc.crcid
		WHERE (_CrcID IS NULL OR crc.crcid=_CrcID) AND (bt.bptype='VEND' OR (bt.bptype!='VEND' AND bpacc.apbalance<>0));
	LOOP
		FETCH cBal INTO _BPID, _BPName, _Balance, _CrcName, _CrcSymbol, _CurrCrcID;
		EXIT WHEN NOT FOUND;

		SELECT INTO __Balance SUM(bpaamt) AS balance 
		FROM bpa m 
		WHERE m.bpid=_BPID AND m.crcid=_CurrCrcID AND reftype IN ('PURC', 'PAY', 'PRET') 
		AND m.bpadate>=_StartDate AND m.bpadate<=__Date;

		IF NOT FOUND THEN
			__Balance:=0;
		END IF;

		_Balance:=_Balance+COALESCE(__Balance,0);

		SELECT INTO _FCNBalance SUM(bpaamt) 
		FROM bpa m JOIN fcn ON m.refno=fcn.fcnno
		WHERE m.bpid=_BPID AND m.crcid=_CurrCrcID 
		AND m.bpadate>=_StartDate AND m.bpadate<=__Date
		AND m.reftype='FCN' AND fcn.dc='C';

		IF NOT FOUND THEN
			_FCNBalance:=0;
		END IF;

		_Balance:=_Balance+COALESCE(_FCNBalance,0);
		

		IF ((_NotEqualZero=TRUE AND _Balance<>0) OR (_NotEqualZero=FALSE)) THEN
			_retval.bpid:=_BPID;
			_retval.bpname:=_BPName;
			_retval.balance:=abs(_Balance);
			_retval.crcname:=_CrcName;
			_retval.crcsymbol:=_CrcSymbol;
	
			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION fPiutangDate(
	_Tipe		INT,
	_Date		DATE,
	_CrcID 		VARCHAR,
	_NotEqualZero	BOOLEAN
) RETURNS SETOF tHutangPiutang AS
$BODY$
DECLARE
	_retval		tHutangPiutang%ROWTYPE;

	_BPID		VARCHAR(15); 
	_BPName		VARCHAR(50); 
	_Balance	NUMERIC(19,4); 
	_CrcName	VARCHAR(50); 
	_CrcSymbol	VARCHAR(5);
	_CurrCrcID	VARCHAR;

	__Date		DATE;

	_PYear		INT2;
	_Month		INT2;
	_Day		INT2;
	_StartDate	DATE;
	
	__Balance	NUMERIC(19,4);
	_FCNBalance	NUMERIC(19,4);
	
	cBal		REFCURSOR;
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month;

	OPEN cBal FOR
		SELECT DISTINCT bp.bpid, bpname, fBPBalanceDate(1, bp.bpid, crc.crcid, _Month, _PYear) AS arbalance, crcname, crcsymbol, crc.crcid  
		FROM bp LEFT JOIN bpacc ON bp.bpid=bpacc.bpid  JOIN bpbptype bt ON bt.bpid=bp.bpid
		JOIN crc ON crc.crcid=bpacc.crcid
		WHERE (_CrcID IS NULL OR crc.crcid=_CrcID) AND (bt.bptype='CUST' OR (bt.bptype!='CUST' AND bpacc.arbalance<>0));
	LOOP
		FETCH cBal INTO _BPID, _BPName, _Balance, _CrcName, _CrcSymbol, _CurrCrcID;
		EXIT WHEN NOT FOUND;

		SELECT INTO __Balance SUM(bpaamt) AS balance FROM bpa m 
		WHERE m.bpid=_BPID AND m.crcid=_CurrCrcID AND reftype IN ('SALE', 'RCV', 'SRET') 
		AND m.bpadate>=_StartDate AND m.bpadate<=__Date;
		
		IF NOT FOUND THEN
			__Balance:=0;
		END IF;

		_Balance:=_Balance+COALESCE(__Balance,0);

		SELECT INTO _FCNBalance SUM(bpaamt) 
		FROM bpa m JOIN fcn ON m.refno=fcn.fcnno 
		WHERE m.bpid=_BPID AND m.crcid=_CurrCrcID 
		AND m.bpadate>=_StartDate AND m.bpadate<=__Date
		AND m.reftype='FCN' AND fcn.dc='D';

		IF NOT FOUND THEN
			_FCNBalance:=0;
		END IF;

		_Balance:=_Balance+COALESCE(_FCNBalance,0);

		IF ((_NotEqualZero=TRUE AND _Balance<>0) OR (_NotEqualZero=FALSE)) THEN
			_retval.bpid:=_BPID;
			_retval.bpname:=_BPName;
			_retval.balance:=_Balance;
			_retval.crcname:=_CrcName;
			_retval.crcsymbol:=_CrcSymbol;

			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


